@@ -63,6 +63,7 @@ from The Open Group.
 #include "Xlcint.h"
 #include <ctype.h>
 #include <X11/Xos.h>
+#include "reallocarray.h"
 
 
 #define	XMAXLIST	256
@@ -112,11 +113,11 @@ _XParseBaseFontNameList(
 	if (!*ptr)
 	    break;
     }
-    if (!(list = (char **) Xmalloc((unsigned)sizeof(char *) * (*num + 1)))) {
+    if (!(list = Xmallocarray((*num + 1), sizeof(char *)))) {
 	Xfree(psave);
 	return (char **)NULL;
     }
-    memcpy((char *)list, (char *)plist, sizeof(char *) * (*num));
+    memcpy((char *)list, (char *)plist, sizeof(char *) * (size_t) (*num));
     *(list + *num) = NULL;
 
     return list;
@@ -130,19 +131,19 @@ copy_string_list(
     char **string_list_ret, **list_src, **list_dst, *dst;
     int length, count;
 
-    if (string_list == NULL || list_count == 0)
+    if (string_list == NULL || list_count <= 0)
 	return (char **) NULL;
 
-    string_list_ret = (char **) Xmalloc(sizeof(char *) * list_count);
+    string_list_ret = Xmallocarray(list_count, sizeof(char *));
     if (string_list_ret == NULL)
 	return (char **) NULL;
 
     list_src = string_list;
     count = list_count;
     for (length = 0; count-- > 0; list_src++)
-	length += strlen(*list_src) + 1;
+	length = length + (int) strlen(*list_src) + 1;
 
-    dst = (char *) Xmalloc(length);
+    dst = Xmalloc(length);
     if (dst == NULL) {
 	Xfree(string_list_ret);
 	return (char **) NULL;
@@ -195,7 +196,7 @@ XCreateFontSet (
     if (oc && def_string) {
 	*def_string = oc->core.default_string;
 	if (!*def_string)
-	    *def_string = "";
+	    *def_string = (char *)"";
     }
 
     if (oc == NULL)
